@@ -1,7 +1,7 @@
 /*
  * downloader.h
  *
- * Copyright (C) 2007 Carillon Information Security Inc.
+ * Copyright (C) 2007-2009 Carillon Information Security Inc.
  *
  * This program and accompanying library is covered by the LGPL v2.1 or later, 
  * please read LICENSE for details.
@@ -18,28 +18,29 @@
 #include <xplc/ptr.h>
 
 
-typedef WvCallback<void, WvStringParm, WvStringParm, WvBuf&, 
-    WvError, void*> DownloadFinishedCb;
+typedef wv::function<void(WvStringParm, WvStringParm, WvBuf&, WvError)> DownloadFinishedCb;
 
 class Downloader
 {
-  public:
-    Downloader(WvStringParm _url, WvHttpPool *_pool, DownloadFinishedCb _cb, 
-               void *_userdata);
+public:
+    Downloader(WvStringParm _url, WvHttpPool *_pool, 
+               DownloadFinishedCb _cb,
+               WvStringParm _method = "GET",
+               WvStringParm _headers = "",
+               WvStream *content_source = NULL);
     virtual ~Downloader();
-    void download();
     bool is_done() { return done; }
 
-  private:
+private:
     WvDynBuf downloadbuf;
-    void download_cb(WvStream &s, void *);
+    void download_cb(WvStream &s);
     void download_closed_cb(WvStream &s);
-
+    void download_ldap();
+    
     WvString url;
     WvHttpPool *pool;
     xplc_ptr<WvBufUrlStream> stream;
     DownloadFinishedCb finished_cb;
-    void *userdata;
     bool done;
     WvLog log;
 };
