@@ -12,6 +12,12 @@
 extern "C" {
 #endif
 
+int pathfinder_app_dbus_verify(const char *appname,
+                               const char *certhex, const char *policy,
+                               const int initial_explicit_policy,
+                               const int initial_policy_mapping_inhibit,
+                               char **errmsg);
+
 int pathfinder_dbus_verify(const char *certhex, const char *policy, 
                            const int initial_explicit_policy, 
                            const int initial_policy_mapping_inhibit,
