@@ -16,34 +16,38 @@
 class PathValidator
 {
   public:
-    typedef WvCallback<void, boost::shared_ptr<WvX509> &, bool, 
-                       WvError, void *> ValidatedCb;
+    typedef wv::function<void(boost::shared_ptr<WvX509> &, bool, 
+                              WvError)> ValidatedCb;
     PathValidator(boost::shared_ptr<WvX509> &_cert,
                   WvStringParm _initial_policy_set_tcl, 
                   uint32_t _validation_flags,
                   boost::shared_ptr<WvX509Store> &_trusted_store,
                   boost::shared_ptr<WvX509Store> &_intermediate_store,
+                  boost::shared_ptr<WvCRLCache> &_crlcache,
                   UniConf &_cfg, 
-                  ValidatedCb _cb, void *_userdata);
+                  ValidatedCb _cb);
     
-    void validate();
+    void validate(bool check_ocsp = true);
 
   private:
-    void path_found_cb(boost::shared_ptr<WvX509Path> &path, WvError err, void *);
-    typedef std::map< std::string, boost::shared_ptr<PathFinder> > PathFinderMap;
-    typedef std::pair< std::string, boost::shared_ptr<PathFinder> > PathFinderPair;
-    PathFinderMap pathfinder_map;
+    void path_found_cb(boost::shared_ptr<WvX509Path> &path, WvError err,
+                       boost::shared_ptr<WvX509> &cert);
 
     boost::shared_ptr<WvX509> cert_to_be_validated;
+    WvX509List certs_to_be_validated;
     WvStringList initial_policy_set;
     uint32_t validation_flags;
 
+    // keep a list of pathfinder objects we create, for reference counting
+    // purposes
+    std::list<boost::shared_ptr<PathFinder> > pathfinder_list;
+
     boost::shared_ptr<WvX509Store> trusted_store;
     boost::shared_ptr<WvX509Store> intermediate_store;
+    boost::shared_ptr<WvCRLCache> crlcache;
 
     UniConf cfg;
     ValidatedCb validated_cb;
-    void *userdata;
 
     WvLog log;
 };
