@@ -1,9 +1,10 @@
 /* -*- Mode: C++ -*-
  * X.509 certificate path management classes.
  *
- * Copyright (C) 2007, Carillon Information Security Inc.
+ * Copyright (C) 2007-2008, Carillon Information Security Inc.
  * 
- * This library is covered by the LGPL v2.1 or later, please read LICENSE for details.
+ * This library is covered by the LGPL v2.1 or later, please read LICENSE for 
+ * details.
  */ 
 
 #include <wvstrutils.h>
@@ -34,7 +35,7 @@ WvString WvX509Path::get_end_entity_ski()
 }
 
 
-void WvX509Path::prepend_cert(boost::shared_ptr<WvX509> &x509)
+void WvX509Path::prepend_cert(shared_ptr<WvX509> &x509)
 {
     log("Prepending cert %s to path.\n", x509->get_subject());
     x509_list.push_front(x509);
@@ -48,9 +49,18 @@ void WvX509Path::append_cert(shared_ptr<WvX509> &x509)
 }
 
 
-void WvX509Path::add_crl(WvStringParm ski, shared_ptr<WvCRL> &crl)
+void WvX509Path::add_crl(WvStringParm subject, shared_ptr<WvCRL> &crl)
 {
-    crl_map.insert(CRLPair(ski.cstr(), crl));
+    log("Adding a CRL for %s.\n", subject);
+    crl_map.insert(CRLPair(subject.cstr(), crl));
+}
+
+
+void WvX509Path::add_ocsp_resp(WvStringParm subject,
+                               shared_ptr<WvOCSPResp> &ocsp)
+{
+    log("Adding an OCSP response for %s.\n", subject);
+    ocsp_map.insert(OCSPRespPair(subject.cstr(), ocsp));
 }
 
 
@@ -63,7 +73,7 @@ void validate_crl(WvX509Store *store, shared_ptr<WvX509> &x509)
 
 void WvX509Path::validate_failed(WvStringParm errstring, WvError &err)
 {
-    log("%s. Failed.\n", errstring);
+    log(WvLog::Error, "%s. Failed.\n", errstring);
     err.seterr(errstring);
 }
 
@@ -78,11 +88,12 @@ bool WvX509Path::validate(shared_ptr<WvX509Store> &trusted_store,
     if (x509_list.size() == 0)
         return true;
 
-    bool check_crls = !(flags & WVX509_SKIP_CRL_CHECK);
+    bool check_revocation = !(flags & WVX509_SKIP_REVOCATION_CHECK);
     bool ignore_missing_crls = (flags & WVX509_IGNORE_MISSING_CRLS);
     bool check_policy = !(flags & WVX509_SKIP_POLICY_CHECK);
     bool initial_explicit_policy = (flags & WVX509_INITIAL_EXPLICIT_POLICY); 
-    bool initial_inhibit_policy_mapping = (flags & WVX509_INITIAL_POLICY_MAPPING_INHIBIT);
+    bool initial_inhibit_policy_mapping = 
+    (flags & WVX509_INITIAL_POLICY_MAPPING_INHIBIT);
 
     int explicit_policy = 0;
     if (!initial_explicit_policy)
@@ -99,10 +110,17 @@ bool WvX509Path::validate(shared_ptr<WvX509Store> &trusted_store,
     // first, find the trust anchor associated with the path. if we can't 
     // find one, we can't continue
     WvString trusted_aki = (*(x509_list.begin()))->get_aki();
-    shared_ptr<WvX509> prev = trusted_store->get(trusted_aki);    
+    WvString trusted_issuer = (*(x509_list.begin()))->get_issuer();
+    shared_ptr<WvX509> prev;
+    if (!!trusted_aki) // look up with aki if we can, more reliable
+        prev = trusted_store->get(trusted_aki);    
+    else
+        prev = trusted_store->get(trusted_issuer);
+
     if (!prev)
     {
-        validate_failed("Trusted root for path not in store", err);
+        validate_failed(WvString("Trusted root for path (%s/%s) not in store", 
+                                 trusted_aki, trusted_issuer), err);
         return false;
     }
 
@@ -128,7 +146,8 @@ bool WvX509Path::validate(shared_ptr<WvX509Store> &trusted_store,
 
         if (!cur->validate())
         {
-            validate_failed(WvString("Certificate '%s' not valid", cur->get_subject()), err);
+            validate_failed(WvString("Certificate '%s' not valid", 
+                                     cur->get_subject()), err);
             return false;
         }
 
@@ -148,38 +167,173 @@ bool WvX509Path::validate(shared_ptr<WvX509Store> &trusted_store,
             return false;
         }
 
-        if (check_crls)
+        // OCSP validation is pretty simple: look it up in the map, make 
+        // sure our current certificate is not revoked, then add the OCSP
+        // responder certificate to our list of extra certificates to be 
+        // validated. note that we also need aki info to make it work.
+        bool validated_ocsp = false;        
+        bool have_aki = !!cur->get_aki();
+        bool have_ski = !!cur->get_ski();
+        if (check_revocation) 
+        {
+            pair<OCSPRespMap::iterator, OCSPRespMap::iterator> iterpair = 
+            ocsp_map.equal_range(cur->get_subject().cstr());
+            if (iterpair.first != iterpair.second)
+            {
+                shared_ptr<WvOCSPResp> resp = (*iterpair.first).second;
+                WvX509 resp_signer = resp->get_signing_cert();
+                
+                WvOCSPResp::Status status = resp->get_status(*cur, *prev);
+                if (status != WvOCSPResp::Good)
+                {
+                    validate_failed(WvString("Certificate %s's OCSP response "
+                                             "does not check out (status: %s)",
+                                             cur->get_subject(), 
+                                             WvOCSPResp::status_str(status)),
+                                    err);
+                    return false;
+                }
+                
+                if (!resp_signer)
+                {
+                    validate_failed(WvString("Certificate %s's OCSP response "
+                                             "does not have a signing "
+                                             "certificate", 
+                                             cur->get_subject()), err);
+                    return false;
+                }
+
+                if (!resp->signedbycert(resp_signer))
+                {
+                    validate_failed(WvString("Certificate %s's OCSP response "
+                                             "is not properly signed by OCSP "
+                                             "response signer",
+                                             cur->get_subject()), err);
+                    return false;
+                }
+
+                bool responder_has_ocsp_signing_key_usage = false;
+                WvStringList ext_key_usage;
+                ext_key_usage.split(resp_signer.get_ext_key_usage(), ";\n");
+                WvStringList::Iter i(ext_key_usage);
+                for (i.rewind(); i.next();)
+                {
+                    if (i() == "OCSP Signing")
+                    {
+                        responder_has_ocsp_signing_key_usage = true;
+                        break;
+                    }
+                }
+                if (!responder_has_ocsp_signing_key_usage)
+                {
+                    validate_failed(WvString("Certificate %s's OCSP responder "
+                                             "does not have OCSP Signing in "
+                                             "its extended key usage",
+                                             cur->get_subject()), err);
+                    return false;
+                }
+                
+                if ((have_ski && resp_signer.get_aki() == cur->get_ski())
+                        ||
+                   (resp_signer.get_issuer() == cur->get_subject()))
+                {
+                    // this is somewhat questionable, but allow it for now:
+                    // some certificates in the wild are the signer of their
+                    // own OCSP responder
+                    log(WvLog::Warning, "Certificate %s's OCSP responder's "
+                        "seems to be signed by the current certificate. This "
+                        "is somewhat questionable.\n");
+                }
+                else if ((have_aki && resp_signer.get_aki() != cur->get_aki())
+                        ||
+                    (resp_signer.get_issuer() != cur->get_issuer()))
+                {
+                    if (have_aki)
+                        validate_failed(WvString("Certificate %s's OCSP "
+                                             "responder's AKI (%s) does not "
+                                             "match own (%s)",
+                                             cur->get_subject(), 
+                                             resp_signer.get_aki(),
+                                             cur->get_aki()), err);
+                    else
+                        validate_failed(WvString("Certificate %s's OCSP "
+                                             "responder's issuer (%s) does "
+                                             "not match own (%s)",
+                                             cur->get_subject(),
+                                             resp_signer.get_issuer(),
+                                             cur->get_issuer()), err);
+                    return false;
+                }
+
+                // validate the cert *UNLESS* the id-pkix-ocsp-nocheck
+                // extension is present. (sigh)
+                if (X509_get_ext_by_NID(resp_signer.get_cert(),
+                                        NID_id_pkix_OCSP_noCheck, -1) < 0)
+                {
+                    extra_certs_to_be_validated.push_back(
+                        shared_ptr<WvX509>(new WvX509(resp_signer)));
+                }
+                else
+                {
+                    log(WvLog::Info, "Not validating the OCSP signing "
+                        "certificate (%s) since it asserts the "
+                        "id-pkix-ocsp-nocheck extension.\n",
+                        resp_signer.get_subject());
+                }
+                validated_ocsp = true;
+            }
+        }
+            
+        // CRL validation is much more involved... we try to follow what's 
+        // laid out in rfc3280 to the letter
+        if (check_revocation && !validated_ocsp)
         {
             pair<CRLMap::iterator, CRLMap::iterator> iterpair = 
-            crl_map.equal_range(cur->get_ski().cstr());
+                crl_map.equal_range(cur->get_subject().cstr());
 
             bool one_valid_crl = false;
             for (CRLMap::iterator j = iterpair.first; j != iterpair.second; j++)
             {
                 shared_ptr<WvCRL> crl = (*j).second;
 
-                shared_ptr<WvX509> crl_signer;
+                // we need to trim spaces and convert to lower case: 
+                // differences in spacing or case shouldn't make a difference 
+                // for validation
+                WvString crl_issuer = strreplace(crl->get_issuer(), " ", "");
+                strlwr(crl_issuer.edit());
+                WvString cert_issuer = strreplace(cur->get_issuer(), " ", "");
+                strlwr(cert_issuer.edit());
                 WvString crl_aki = crl->get_aki();
-                if (!crl_aki)
-                {
-                    log("Certificate revocation list for %s has no AKI.\n",
-                        cur->get_subject());
-                    continue;
-                }
-                crl_signer = trusted_store->get(crl_aki);
+                bool crl_signer_untrusted = false;
+
+                shared_ptr<WvX509> crl_signer;
+                if (prev->get_ski() == crl_aki)
+                    crl_signer = prev;
+                if (!crl_signer && prev->get_subject() == crl_issuer)
+                    crl_signer = prev;
+                if (!crl_signer)                
+                    crl_signer = trusted_store->get(crl_aki);                
+                // as a last resort, search in the intermediate store for a 
+                // crl signer. this crl signer will need to be validated
+                // seperately
                 if (!crl_signer)
+                {
                     crl_signer = intermediate_store->get(crl_aki);
+                    crl_signer_untrusted = true;
+                }
+
                 if (!crl_signer)
                 {
-                    log("Can't find CRL signer for %s's CRL in intermediate "
-                        "or trusted store.\n", cur->get_subject());
+                    log(WvLog::Info, "CRL signer is not the certificate's "
+                        "signer, nor can we find it in the trusted store.\n", 
+                        cur->get_subject());
                     continue;
                 }
 
                 if (crl->validate(*(crl_signer.get())) != WvCRL::VALID)
                 {
-                    log("Certificate revocation list for %s is not valid.\n", 
-                        cur->get_subject());
+                    log(WvLog::Info, "Certificate revocation list for %s is "
+                        "not valid.\n", cur->get_subject());
                     continue;
                 }
 
@@ -188,35 +342,24 @@ bool WvX509Path::validate(shared_ptr<WvX509Store> &trusted_store,
                 // issuer's name of the crl should match the issuer name 
                 // of the certificate we are processing.
                 
-                // we need to trim spaces and convert to lower case: differences in
-                // spacing or case shouldn't make a difference for validation
-                WvString crl_issuer = strreplace(crl->get_issuer(), " ", "");
-                strlwr(crl_issuer.edit());
-                WvString cert_issuer = strreplace(cur->get_issuer(), " ", "");
-                strlwr(cert_issuer.edit());
-
                 if (crl_issuer != cert_issuer)
                 {
-                    log("CRL's issuer (%s) does not match certificate's "
-                        "issuer (%s).\n", crl_issuer, cert_issuer);
+                    log(WvLog::Info, "CRL's issuer (%s) does not match "
+                        "certificate's issuer (%s).\n", crl_issuer, 
+                        cert_issuer);
                     continue;
                 }
-                else
-                {
-                    one_valid_crl = true;
 
-                    // if the aki of the crl issuer does not match that of the
-                    // previous certificate, it will need validation in order 
-                    // for this path to be validated
-                    if (crl_signer->get_ski() != prev->get_ski())
-                        extra_certs_to_be_validated.push_back(crl_signer);
-
-                }
+                // if we got this far, our CRL is valid. however, we may need
+                // to validate our CRL signer if it's untrusted
+                one_valid_crl = true;
+                if (crl_signer_untrusted)
+                    extra_certs_to_be_validated.push_back(crl_signer);
 
                 if (crl->isrevoked(*(cur.get())))
                 {
-                    log("Certificate %s is revoked according to CRL.\n", 
-                        cur->get_subject());
+                    log(WvLog::Error, "Certificate %s is revoked according to "
+                        "CRL.\n", cur->get_subject());
                     return false;
                 }
             }
@@ -432,9 +575,11 @@ bool WvX509Path::validate(shared_ptr<WvX509Store> &trusted_store,
 
         policy_tree.intersection(initial_policy_set, policy_level);
         if (policy_tree.isnull())
+        {
+            validate_failed("Policy tree is null during policy "
+                            "checking", err);
             return false;
-
-
+        }
     }
 
     return true;
